'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _babelPluginSyntaxFlow = require('babel-plugin-syntax-flow');

var _babelPluginSyntaxFlow2 = _interopRequireDefault(_babelPluginSyntaxFlow);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _lodash = require('lodash.snakecase');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.camelcase');

var _lodash4 = _interopRequireDefault(_lodash3);

var _babelAddFlowComments = require('babel-add-flow-comments');

var _babelAddFlowComments2 = _interopRequireDefault(_babelAddFlowComments);

var _s2sUtils = require('s2s-utils');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import blog from 'babel-log'

const constantCase = str => (0, _lodash2.default)(str).toUpperCase();

const builders = {
  actionCreater: (0, _s2sUtils.template)(`export const NAME = (PARAMS): Action => {
    return VALUE;
  }`),
  importConstants: (0, _s2sUtils.template)(`import * as constants from './constants'`)
};

function createActionCreater(name, props, params) {
  const typeIdentifier = t.identifier('type');
  return builders.actionCreater({
    NAME: t.identifier((0, _lodash4.default)(name)),
    TYPE: t.identifier(name),
    VALUE: t.objectExpression([t.objectProperty(typeIdentifier, t.memberExpression(t.identifier('constants'), t.identifier(constantCase(name)))), ...props]),
    PARAMS: params
  });
}

exports.default = () => {
  return {
    inherits: _babelPluginSyntaxFlow2.default,
    name: 's2s-action-creater',
    visitor: {
      Program: {
        exit(programPath) {
          const imports = [];
          const typeNames = [];
          const funcs = [];

          programPath.traverse({
            ImportDeclaration(path) {
              imports.push(path.node);
            },
            TypeAlias(path) {
              const name = path.get('id').node.name;

              if (name === 'Action') {
                return;
              }
              typeNames.push(name);

              const params = [];
              const props = [];

              for (const _ref of path.get('right').node.properties) {
                const { key, value } = _ref;

                if (key.name !== 'type') {
                  const p = t.identifier(key.name);

                  props.push(t.objectProperty(p, p, false, true));

                  p.typeAnnotation = t.typeAnnotation(value);
                  params.push(p);
                }
              }

              funcs.push(createActionCreater(name, props, params));
              funcs.push(t.noop());
            }
          });

          const typeImport = (0, _s2sUtils.createImportDeclaration)('Action', `./types`);

          typeImport.importKind = 'type';

          programPath.node.body = [typeImport, ...imports, t.noop(), ...funcs];

          (0, _babelAddFlowComments2.default)(programPath);
        }
      }
    }
  };
};